@@ -1,4 +1,4 @@
-// Copyright © 2016-present Bjørn Erik Pedersen <bjorn.erik.pedersen@gmail.com>.
+// Copyright 2024 Bjørn Erik Pedersen <bjorn.erik.pedersen@gmail.com>.
 //
 // Use of this source code is governed by an MIT-style
 // license that can be found in the LICENSE file.
@@ -31,7 +31,7 @@ func TestMap(t *testing.T) {
 		err error
 	)
 
-	if gr, err = Map(repository, revision); err != nil {
+	if gr, err = Map(Options{Repository: repository, Revision: revision}); err != nil {
 		t.Fatal(err)
 	}
 
@@ -81,8 +81,8 @@ func assertFile(
 	expectedAbbreviatedHash,
 	expectedHash,
 	expectedAuthorDate,
-	expectedCommitDate string) {
-
+	expectedCommitDate string,
+) {
 	var (
 		gi *GitInfo
 		ok bool
@@ -113,6 +113,68 @@ func assertFile(
 	}
 }
 
+func TestCommitMessage(t *testing.T) {
+	var (
+		gm  GitMap
+		gr  *GitRepo
+		err error
+	)
+
+	if gr, err = Map(Options{Repository: repository, Revision: "HEAD"}); err != nil {
+		t.Fatal(err)
+	}
+
+	gm = gr.Files
+
+	assertMessage(
+		t, gm,
+		"testfiles/d1/d1.txt",
+		"Change the test files",
+		"To trigger a test variant.",
+	)
+
+	assertMessage(
+		t, gm,
+		"testfiles/r3.txt",
+		"Edit testfiles/r3.txt",
+		"Multiline\n\ncommit body.",
+	)
+
+	assertMessage(
+		t, gm,
+		"testfiles/amended.txt",
+		"Add testfile with different author/commit date",
+		"",
+	)
+}
+
+func assertMessage(
+	t *testing.T,
+	gm GitMap,
+	filename,
+	expectedSubject,
+	expectedBody string,
+) {
+	t.Helper()
+
+	var (
+		gi *GitInfo
+		ok bool
+	)
+
+	if gi, ok = gm[filename]; !ok {
+		t.Fatal(filename)
+	}
+
+	if gi.Subject != expectedSubject {
+		t.Fatalf("Incorrect commit subject. Expected:\n%q\nGot:\n%q", expectedSubject, gi.Subject)
+	}
+
+	if gi.Body != expectedBody {
+		t.Fatalf("Incorrect commit body. Expected:\n%q\nGot:\n%q", expectedBody, gi.Body)
+	}
+}
+
 func TestActiveRevision(t *testing.T) {
 	var (
 		gm  GitMap
@@ -120,7 +182,7 @@ func TestActiveRevision(t *testing.T) {
 		err error
 	)
 
-	if gr, err = Map(repository, "HEAD"); err != nil {
+	if gr, err = Map(Options{Repository: repository, Revision: "HEAD"}); err != nil {
 		t.Fatal(err)
 	}
 
@@ -138,12 +200,11 @@ func TestActiveRevision(t *testing.T) {
 func TestGitExecutableNotFound(t *testing.T) {
 	defer initDefaults()
 	gitExec = "thisShouldHopefullyNotExistOnPath"
-	gi, err := Map(repository, revision)
+	gi, err := Map(Options{Repository: repository, Revision: revision})
 
-	if err != GitNotFound || gi != nil {
+	if err != ErrGitNotFound || gi != nil {
 		t.Fatal("Invalid error handling")
 	}
-
 }
 
 func TestEncodeJSON(t *testing.T) {
@@ -156,7 +217,7 @@ func TestEncodeJSON(t *testing.T) {
 		filename = "README.md"
 	)
 
-	if gr, err = Map(repository, revision); err != nil {
+	if gr, err = Map(Options{Repository: repository, Revision: revision}); err != nil {
 		t.Fatal(err)
 	}
 
@@ -167,20 +228,19 @@ func TestEncodeJSON(t *testing.T) {
 	}
 
 	b, err := json.Marshal(&gi)
-
 	if err != nil {
 		t.Fatal(err)
 	}
 
 	s := string(b)
 
-	if s != `{"hash":"0b830e458446fdb774b1688af9b402acf388d6ab","abbreviatedHash":"0b830e4","subject":"Add some more to README","authorName":"Bjørn Erik Pedersen","authorEmail":"bjorn.erik.pedersen@gmail.com","authorDate":"2016-07-22T21:40:27+02:00","commitDate":"2016-07-22T21:40:27+02:00"}` {
+	if s != `{"hash":"0b830e458446fdb774b1688af9b402acf388d6ab","abbreviatedHash":"0b830e4","subject":"Add some more to README","authorName":"Bjørn Erik Pedersen","authorEmail":"bjorn.erik.pedersen@gmail.com","authorDate":"2016-07-22T21:40:27+02:00","commitDate":"2016-07-22T21:40:27+02:00","body":""}` {
 		t.Errorf("JSON marshal error: \n%s", s)
 	}
 }
 
 func TestGitRevisionNotFound(t *testing.T) {
-	gi, err := Map(repository, "adfasdfasdf")
+	gi, err := Map(Options{Repository: repository, Revision: "adfasdfasdf"})
 
 	// TODO(bep) improve error handling.
 	if err == nil || gi != nil {
@@ -189,7 +249,7 @@ func TestGitRevisionNotFound(t *testing.T) {
 }
 
 func TestGitRepoNotFound(t *testing.T) {
-	gi, err := Map("adfasdfasdf", revision)
+	gi, err := Map(Options{Repository: "adfasdfasdf", Revision: revision})
 
 	// TODO(bep) improve error handling.
 	if err == nil || gi != nil {
@@ -203,11 +263,11 @@ func TestTopLevelAbsPath(t *testing.T) {
 		err error
 	)
 
-	if gr, err = Map(repository, revision); err != nil {
+	if gr, err = Map(Options{Repository: repository, Revision: revision}); err != nil {
 		t.Fatal(err)
 	}
 
-	expected := "/bep/gitmap"
+	expected := "/gitmap"
 
 	if !strings.HasSuffix(gr.TopLevelAbsPath, expected) {
 		t.Fatalf("Expected to end with %q got %q", expected, gr.TopLevelAbsPath)
@@ -216,7 +276,7 @@ func TestTopLevelAbsPath(t *testing.T) {
 
 func BenchmarkMap(b *testing.B) {
 	for i := 0; i < b.N; i++ {
-		_, err := Map(repository, revision)
+		_, err := Map(Options{Repository: repository, Revision: revision})
 		if err != nil {
 			b.Fatalf("Got error: %s", err)
 		}
